@@ -1,17 +1,17 @@
 package Mail::GPG;
 
-# $Id: GPG.pm,v 1.26 2011-08-03 08:53:43 joern Exp $
-
-$VERSION = "1.0.7";
+$VERSION = "1.0.8";
 
 use strict;
 use Carp;
 use IO::Handle;
+use Data::Dumper ();
 use GnuPG::Interface;
 use MIME::Parser;
 use Mail::GPG::Result;
 use Mail::Address;
 use File::Temp;
+use List::MoreUtils ();
 
 $Mail::GPG::SKIP_EVENT = $ENV{MAIL_GPG_SKIP_EVENT}
     unless defined $Mail::GPG::SKIP_EVENT;
@@ -51,7 +51,7 @@ sub new {
     my  ($no_strict_7bit_encoding, $use_long_key_ids) =
     @par{'no_strict_7bit_encoding','use_long_key_ids'};
 
-    $debug_dir               ||= File::Spec->tmpdir;
+    $debug_dir               ||= $ENV{DUMPDIR} || File::Spec->tmpdir . '/mail-gpg-test';
     $gnupg_hash_init         ||= {};
     $digest                  ||= "RIPEMD160";
     $gpg_call                ||= "gpg";
@@ -418,10 +418,163 @@ sub perform_multiplexed_gpg_io_event {
     1;
 }
 
+
+
+#-- Addresses bug:     https://rt.cpan.org/Public/Bug/Display.html?id=103828  (here)
+#-- Possibly also:     https://rt.cpan.org/Public/Bug/Display.html?id=81082   (here)
+#                  and https://rt.cpan.org/Public/Bug/Display.html?id=80500   (in Makefile.PL)
+
+sub _parse_key_list {
+  my $self = shift;
+  my ($output_stdout, %par) = @_;
+  my  ($coerce, $debug, $verbose) =
+  @par{'coerce','debug','verbose'};
+
+  #-- grab key ID's and emails from output (backward compatible to gpg 1.x)
+  #
+  #  Example:
+  #  search request: "--list-keys --with-colons 6C187D0F196ED9E3"
+  #  format        : see /usr/share/doc/packages/gpg2/DETAILS
+  #
+  #--OLD sample: gpg (GnuPG) 1.4.11
+  #   tru:t:1:1431088683:0:3:1:5
+  #   pub:-:1024:17:062F00DAE20F5035:2004-02-10:::-:Jrn Reder Mail\x3a\x3aGPG Test Key <mailgpg@localdomain>::scaESCA:
+  #   sub:-:1024:16:6C187D0F196ED9E3:2004-02-10::::::e:
+  #
+  #   expected output: (062F00DAE20F5035, 'Jrn Reder Mail::GPG Test Key <mailgpg@localdomain>')
+  #
+  #--NEW sample: gpg (GnuPG) 2.0.22
+  #  tru:t:1:1429473192:0:3:1:5
+  #  pub:-:1024:17:062F00DAE20F5035:1076425915:::-:::scaESCA:
+  #  uid:-::::1076425915::588869ADE077B8FB05788A99565AEED15AED8231::Jrn Reder Mail\x3a\x3aGPG Test Key <mailgpg@localdomain>:
+  #  sub:-:1024:16:6C187D0F196ED9E3:1076425917::::::e:
+  #
+  #  expected output: (062F00DAE20F5035, 'Jrn Reder Mail::GPG Test Key <mailgpg@localdomain>',
+  #                    6C187D0F196ED9E3, 'Jrn Reder Mail::GPG Test Key <mailgpg@localdomain>')
+  #
+  #  PERLBOTIX<tt>cpan.org / May, 2015
+
+  my @result;
+
+  #-- needed for utf8 handling
+  require Encode if $] >= 5.008;
+
+  $output_stdout .= "\nFLUSH\n"; #-- we add this token to trigger flushing of the last record
+
+  #-- these values are valid per "paragraph" (from pub: to pub:)
+  my @ids;                       #-- list of potential IDs (pub, sub)
+  my @emails;                    #-- list of potential email adresses (uid)
+  my $gpg2_mode = 0;             #-- auto-detect OLD/NEW format: true if NEW (gpg2) format
+
+  #-- parse output line by line
+  #--  OLD-format (gpg 1.x): simulate old behaviour
+  #--  NEW-format (gpg 2.x): create all combinations of valid key-IDs and emails; return (pub) key-ID first,
+  #    so the result stays backward compatible
+
+  while (  $output_stdout =~ m!^((\w+):?.*?)[\r\n]+!mg  ) {
+
+    my ($line, $tag) = ($1, $2);
+    my @fields       = split /:/, $line, -1;
+
+    warn("\nFields: ", join(", ", map { "($_)" } @fields)) if $debug;
+
+    #-- skip entries that are expired or otherwise invalid
+    if (     ( $fields[1]  and  $fields[1]  =~ /^([eird])$/ )
+          or ( $fields[11] and  $fields[11] =~ /([D])/      )
+       ) { #-- cannot use key (invalid/expired/revoked/[dD]isabled)
+      carp "Skipping key invalidated by '$1'-flag: ($line)" if $verbose or $debug;
+      next;
+    }
+
+    #-- a further 'pub' marks a new paragraph or the end of the current paragraph
+    #-- a 'FLUSH' marks the end of the the whole text
+    if ( $tag eq 'pub'  or  $tag eq 'FLUSH' ) {
+
+      #-- flush intermediate or final results
+      if ( @ids  and  @emails ) {
+
+        @emails = List::MoreUtils::uniq @emails; #-- uniq preserves order
+
+        for ( @emails ) {
+          #-- $key_mail is quoted C-style (e.g. \x3a is a colon)
+          s/\\x(..)/chr(hex($1))/eg;
+
+          #-- tell Perl that this variable is utf8 encoded
+          #-- (if Perl version is 5.8.0 or greater)
+          Encode::_utf8_on($_) if $] >= 5.008;
+        }
+
+        @ids = List::MoreUtils::uniq @ids;
+        if ( !$self->get_use_long_key_ids ) {
+          $_ = substr($_,-8)  for ( @ids );
+        }
+
+        #-- finally, fill result array with all combinations of key-ids and emails (legacy), found in the previous paragraph
+        for my $id ( @ids ) {
+
+          if ( $coerce ) {
+            push @result, $id, [ @emails ]; #-- "modern" interface
+          }
+          else {
+            for my $m ( @emails ) {
+              push @result, ($id, $m); #-- legacy
+            }
+          }
+        }
+
+      } elsif ( @ids  or  @emails ) {
+        carp "Incomplete key data - key is probably invalid?  (@ids) x (@emails)!";
+      }
+
+      @ids    = ();
+      @emails = ();
+      last if $fields[1] eq 'FLUSH';
+
+
+      #-- parse 'new' format (pub:..;  next: uid:..., uid:..., sub:... )
+      if ( $fields[9] =~ /^\s*$/ ) {
+        $gpg2_mode = 1;
+        push @ids, $fields[4];
+      }
+      #-- parse 'old' format (pub:)
+      elsif ( $fields[9] =~ /<[^>]+>/ ) {
+        $gpg2_mode = 0;
+        push @ids, $fields[4];
+        push @emails, $fields[9];
+      }
+      else {
+        die "Cannot parse: ($line)";
+      }
+
+    } #-- 'pub' & 'FLUSH' handled obove
+
+    #-- handle 'sub' entries  /  extract key-id
+    elsif ( $tag eq 'sub'  and  $gpg2_mode  and   $fields[4]) {
+      push @ids, $fields[4];
+    }
+
+    #-- handle 'uid' entries  /  extract email
+    elsif ( $tag eq 'uid'  and  $gpg2_mode  and   $fields[9] =~ /<[^>]+>/ ) {
+      push @emails, $fields[9];
+    }
+
+    #-- ignore anything else
+    else {
+      warn "Ignoring line [gpg2_mode=$gpg2_mode] --  '$line'"  if $debug;
+    }
+
+  } #-- loop over output_stdout
+
+  warn  Data::Dumper->Dump( [ \@result], [qw(RESULT_AS_REF)] ) if $debug;
+
+  return @result;
+}
+
+
 sub query_keyring {
     my $self     = shift;
     my %par      = @_;
-    my ($search) = $par{'search'};
+    my ($search, $debug) = @par{'search','debug'};
 
     #-- ignore any PIPE signals, in case of gpg exited
     #-- early before we fed our data into it.
@@ -461,38 +614,21 @@ sub query_keyring {
     #-- wait on gpg exit
     waitpid $pid, 0;
 
-    #-- needed for utf8 handling
-    require Encode if $] >= 5.008;
-
-    #-- grab key ID's and emails from output
-    my @result;
-    while (
-        $output_stdout =~ m!^pub:[^:]*:[^:]*:[^:]*:([^:]*):[^:]*:
-				         [^:]*:[^:]*:[^:]*:([^:]*)!mgx
-        ) {
-
-        #-- Field 4 and 9 are key-ID and email address
-        my ( $key_id, $key_mail ) = ( $1, $2 );
-
-        #-- Shorten key id?
-        if ( !$self->get_use_long_key_ids ) {
-            ($key_id) = ( $key_id =~ /(........)$/ );
-        }
-
-        #-- $key_mail is quoted C-style (e.g. \x3a is a colon)
-        $key_mail =~ s/\\x(..)/chr(hex($1))/eg;
-
-        #-- tell Perl that this variable is utf8 encoded
-        #-- (if Perl version is 5.8.0 or greater)
-        Encode::_utf8_on($key_mail) if $] >= 5.008;
-
-        #-- fill result array
-        push @result, $key_id, $key_mail;
+    if ( $debug ) {
+      warn "LIST_KEYS(CMD)    -- --list-keys --with-colons $search\n";
+      warn "LIST_KEYS(STDERR) -- search for ($search):\n$output_stderr\n";
+      warn "LIST_KEYS(STDOUT) -- search for ($search):\n$output_stdout\n";
     }
 
+    my @result = $self->_parse_key_list( $output_stdout , %par);
+
     #-- return result: undef if nothing found, first key-id if
     #-- a scalar is requested, all entries suitable for a hash
     #-- slurp if an array is requested
+
+    # Compatibility note: The first id is always the id of the 'pub' entry, even for a subkey-hit.
+    #                     gpgv2: The result should not be used to initialise a hash, since some emails will be clobbered.
+    #                     We need another interface here. Curently Mail::GPG uses only the first entry...
     return            if not @result;
     return $result[0] if not wantarray;
     return @result;
@@ -1666,7 +1802,9 @@ that is OpenPGP MIME and traditional armor signed/encrypted mails.
   Perl              >= 5.00503
   MIME-tools        >= 5.419
   MIME::QuotedPrint >= 2.20  (part of MIME-Base64 distribution)
-  GnuPG::Interface  >= 0.33  (optionally with shipped patch applied)
+  GnuPG::Interface  >= 0.52
+  Event             >= 1.24
+  List::MoreUtils   >= 0.410
 
 Since version 1.0.7 the Event module is loaded when present. This
 gives a small performance boost when handling big entities. If you
@@ -1679,18 +1817,6 @@ before loading Mail::GPG.
 
 =head1 INSTALLATION
 
-Before you install Mail::GPG you may want to apply the shipped
-GnuPG::Interface patch. It just fixes a warning which is throwed
-on any keyring inspection. This is a known problem and reported
-to the author, hopefully it will be fixed upstream soon:
-
-  % tar xvfz GnuPG-Interface-0.33.tar.gz
-  % cd GnuPG-Interface-0.33
-  % patch -p1 ../Mail-GPG.x.xx/patches/GnuPG-Interface-0.33.tru-record-type.txt
-  % perl Makefile.PL
-  % make test
-  % make install
-
 Then install Mail::GPG
 
   % cd ../Mail-GPG-x.xx
@@ -2228,6 +2354,7 @@ The entity to inspect.
   %result              = $mg->query_keyring ( search => $search );
   $key_id              = $mg->query_keyring ( search => $search );
   ($key_id, $key_mail) = $mg->query_keyring ( search => $search );
+  %mails_by_key_id     = $mg->query_keyring ( search => $search, coerce => 1 );
 
 Searches the keyring for a key id or email address.
 In list context a subsequent list of key id and mail
@@ -2249,14 +2376,50 @@ to care about this ;)
 If you use the module with older Perl versions you need
 to handle utf8 encoded data yourself.
 
+=head3 coerce option & GPGv2
+
+When using GPGv2, the current implementation tries to
+simulate the former behaviour. The first entry will be
+the PK-ID (pub: not sub:). The second entry will be the first
+valid (i.e. not expired, not revoked, etc.) email entry.
+
+If there are sub-keys (sub:) or more email entries (uid:), all
+combinations are returned and the user needs to further filter the
+result. Since there may be many emails assigned to a given key-id,
+assigning to a hash will probably clobber previous id/email entries.
+
+In order to allow assignment to a hash again, the parameter
+C<coerce =E<gt> 1> can be used. When active, a list of pairs is
+returned where the first item is the key-id and the
+second item is an array-ref to a list of email addresses.
+
+ %result = $mg->query_keyring ( search => $search, coerce => 1 );
+ # (
+ #   '999F00DAE20F5123',  [ 'Valid Test Key   <userX@localdomain>',
+ #                          'Valid Test Key   <new@localdomain>' ],
+ #   'CAFE333F196ED333',  [ 'Valid Test Key   <userX@localdomain>',
+ #                          'Valid Test Key   <new@localdomain>' ]
+ # )
+
 =over 4
 
 =item search
 
 Key id or email address to query for.
 
+=item coerce
+
+When true, returns all emails belonging to a key as an array-reference.
+(patched version only!)
+
 =back
 
+
+Tests: All temporary files are created in directory F</tmp/mail-gpg-test> now.
+The environment variable C<DUMPDIR> can be used to select an alternative directory.
+
+
+
 =head2 get_key_trust
 
   $trust = $mg->get_key_trust (
