@@ -1,6 +1,6 @@
 package Mail::GPG::Test;
 
-# $Id: Test.pm,v 1.7 2009-05-30 13:56:20 joern Exp $
+# $Id: Test.pm,v 1.6 2006/11/18 08:48:28 joern Exp $
 
 use strict;
 
@@ -14,6 +14,18 @@ use File::Temp qw(tempdir);
 
 my $TIMEIT = 0;
 
+our $DUMPDIR;
+
+BEGIN {
+  $DUMPDIR  = $ENV{DUMPDIR} // File::Spec->tmpdir . '/mail-gpg-test';
+
+  if (not -d $DUMPDIR ) {
+    File::Path::make_path($DUMPDIR) or die "Cannot create '$DUMPDIR' - $!";
+  }
+}
+
+
+
 my $has_encode = eval { require Encode; 1 };
 
 sub get_gpg_home_dir		{ shift->{gpg_home_dir}			}
@@ -111,7 +123,7 @@ sub print_parse_entity {
 
     my ( $fh, $file ) = File::Temp::tempfile(
         'mgpgXXXXXXXX',
-        DIR    => File::Spec->tmpdir,
+        DIR    => $DUMPDIR,
         UNLINK => 1,
     );
 
@@ -183,7 +195,7 @@ sub sign_test {
     );
 
     if ( $ENV{DUMPFILES} ) {
-        my $tmp_file = "/tmp/$method-$encoding-"
+        my $tmp_file = "$DUMPDIR/$method-$encoding-"
             . ( $attach  ? "attach"  : "noattach" ) . "-"
             . ( $invalid ? "invalid" : "valid" );
 
@@ -210,7 +222,7 @@ sub sign_test {
             ok( 0, "$test_name: mail_sref sign check failed" );
             return;
         }
-        my $tmp_file = "/tmp/Mail-GPG-Test-$$.txt";
+        my $tmp_file = "$DUMPDIR/Mail-GPG-Test-$$.txt";
         open( TMP, "+>$tmp_file" ) or die "can't write $tmp_file";
         print TMP $signed_entity_string;
         if ( !Mail::GPG->is_signed_quick( mail_fh => \*TMP ) ) {
@@ -303,14 +315,22 @@ sub enc_test {
         = $mg->get_decrypt_key( entity => $parsed_entity, );
 
     if ($has_encode) {
-        if (   $dec_key_id ne $self->get_key_id
-            or $dec_key_mail ne $self->get_key_mail ) {
-            ok( 0,
-                      "Decryption key or email wrong: "
-                    . "$dec_key_id=="
-                    . $self->get_key_id );
-            return;
-        }
+      if ( $dec_key_id ne $self->get_key_id ) {
+        ok( 0,
+            "Decryption key wrong: "
+            . "$dec_key_id=="
+            . $self->get_key_id
+          );
+        return;
+      }
+      if ( $dec_key_mail ne $self->get_key_mail ) {
+        ok( 0,
+            "Decryption email wrong: "
+            . "$dec_key_mail=="
+            . $self->get_key_mail
+          );
+        return;
+      }
     }
     else {
         if ( $dec_key_id ne $self->get_key_id ) {
@@ -327,7 +347,7 @@ sub enc_test {
 
     if ( $ENV{DUMPFILES} ) {
         my $tmp_file
-            = "/tmp/$method-$encoding-" . ( $attach ? "attach" : "noattach" );
+            = "$DUMPDIR/$method-$encoding-" . ( $attach ? "attach" : "noattach" );
 
         open( SEND, ">$tmp_file.send" );
         open( RETR, ">$tmp_file.retr" );
@@ -468,15 +488,24 @@ print "$dur !\n";
 }
 
     if ($has_encode) {
-        if (   $dec_key_id ne $self->get_key_id
-            or $dec_key_mail ne $self->get_key_mail ) {
-            ok( 0,
-                      "Decryption key or email wrong: "
-                    . "$dec_key_id=="
-                    . $self->get_key_id );
-            return;
-        }
-    }
+      if ( $dec_key_id ne $self->get_key_id ) {
+        ok( 0,
+            "Decryption - key wrong: "
+            . "$dec_key_id=="
+            . $self->get_key_id );
+        return;
+      }
+
+      if ( $dec_key_mail ne $self->get_key_mail ) {
+        ok( 0,
+            "Decryption - email wrong: "
+            . "$dec_key_mail=="
+            . $self->get_key_mail );
+        return;
+      }
+
+
+      }
     else {
         if ( $dec_key_id ne $self->get_key_id ) {
             ok( 0,
